<?php
/*
 * LatePoint Addons Framework
 * Copyright (c) 2021-2023 TechXela (https://codecanyon.net/user/tech-xela). All Rights Reserved.
 *
 * LICENSE
 * -------
 * This software is furnished under license(s) and may be used and copied only in accordance with the terms of such
 * license(s) along with the inclusion of the above copyright notice. If you purchased an item through CodeCanyon, in
 * which this software came included, please read the full license(s) at: https://codecanyon.net/licenses/standard
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'TechXelaLatePointUtilHelper' ) ) {

	final class TechXelaLatePointUtilHelper {

		public static function ellipseString( $string, $length ): string {
			return substr( $string, 0, $length ) . ( strlen( $string ) > $length ? '...' : '' );
		}

		public static function extractIdsAndQuantities( $rawIdsAndQuantities = [] ): array {
			$rawIdsAndQuantities   = ( is_array( $rawIdsAndQuantities ) ) ? $rawIdsAndQuantities : explode( ',', $rawIdsAndQuantities );
			$cleanIdsAndQuantities = [];
			if ( is_array( $rawIdsAndQuantities ) ) {
				foreach ( $rawIdsAndQuantities as $service_extra_id ) {
					list( $id, $quantity ) = array_pad( explode( ':', $service_extra_id ), 2, 1 );
					if ( is_numeric( $id ) && is_numeric( $quantity ) ) {
						$cleanIdsAndQuantities[ $id ] = $quantity;
					}
				}
			}

			return $cleanIdsAndQuantities;
		}

		public static function emptyRecursive( $value ): bool {
			if ( is_array( $value ) ) {
				$empty = true;
				array_walk_recursive( $value, function ( $item ) use ( &$empty ) {
					$empty = $empty && empty( $item );
				} );
			} else {
				$empty = empty( $value );
			}

			return $empty;
		}

		public static function wrapElementInDoubleCurlyBraces( $element ): string {
			return "{{{$element}}}";
		}

		public static function wrapElementsInDoubleCurlyBraces( array $elements ): array {
			return array_map( function ( $element ) {
				return self::wrapElementInDoubleCurlyBraces( $element );
			}, $elements );
		}

		public static function getIPNatively( $getHostByAddr = false ) {
			foreach (
				array(
					'HTTP_CLIENT_IP',
					'HTTP_X_FORWARDED_FOR',
					'HTTP_X_FORWARDED',
					'HTTP_X_CLUSTER_CLIENT_IP',
					'HTTP_FORWARDED_FOR',
					'HTTP_FORWARDED',
					'REMOTE_ADDR'
				) as $key
			) {
				if ( array_key_exists( $key, $_SERVER ) === true ) {
					foreach ( array_map( 'trim', explode( ',', $_SERVER[ $key ] ) ) as $ip ) {
						if ( filter_var( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE ) !== false ) {
							if ( $getHostByAddr === true ) {
								return getHostByAddr( $ip );
							} else {
								return $ip;
							}
						}
					}
				}
			}

			return false;
		}

		public static function getIpRemotely() {
			$wp_args            = array( 'method' => 'GET' );
			$wp_args['timeout'] = 30;
			$result             = wp_remote_request( 'https://ipecho.net/plain', $wp_args );
			if ( is_wp_error( $result ) ) {
				return false;
			}

			return $result['body'];
		}

		/**
		 * Parse a query string into an associative array.
		 *
		 * If multiple values are found for the same key, the value of that key
		 * value pair will become an array. This function does not parse nested
		 * PHP style arrays into an associative array (e.g., `foo[a]=1&foo[b]=2`
		 * will be parsed into `['foo[a]' => '1', 'foo[b]' => '2'])`.
		 *
		 * @param string $str Query string to parse
		 * @param int|bool $urlEncoding How the query string is encoded
		 *
		 * @return array
		 */
		public static function parseQuery( $str, $urlEncoding = true ) {
			$result = [];

			if ( $str === '' ) {
				return $result;
			}

			if ( $urlEncoding === true ) {
				$decoder = function ( $value ) {
					return rawurldecode( str_replace( '+', ' ', $value ) );
				};
			} elseif ( $urlEncoding === PHP_QUERY_RFC3986 ) {
				$decoder = 'rawurldecode';
			} elseif ( $urlEncoding === PHP_QUERY_RFC1738 ) {
				$decoder = 'urldecode';
			} else {
				$decoder = function ( $str ) {
					return $str;
				};
			}

			foreach ( explode( '&', $str ) as $kvp ) {
				$parts = explode( '=', $kvp, 2 );
				$key   = $decoder( $parts[0] );
				$value = isset( $parts[1] ) ? $decoder( $parts[1] ) : null;
				if ( ! isset( $result[ $key ] ) ) {
					$result[ $key ] = $value;
				} else {
					if ( ! is_array( $result[ $key ] ) ) {
						$result[ $key ] = [ $result[ $key ] ];
					}
					$result[ $key ][] = $value;
				}
			}

			return $result;
		}

		/**
		 * Returns all traits used by a class, its parent classes and trait of their traits.
		 *
		 * @param object|string $class
		 *
		 * @return array
		 */
		public static function classUsesRecursive( $class ): array {
			if ( is_object( $class ) ) {
				$class = get_class( $class );
			}

			$results = [];

			foreach ( array_reverse( class_parents( $class ) ) + [ $class => $class ] as $class ) {
				$results += self::traitUsesRecursive( $class );
			}

			return array_unique( $results );
		}

		/**
		 * Returns all traits used by a trait and its traits.
		 *
		 * @param string $trait
		 *
		 * @return array
		 */
		public static function traitUsesRecursive( $trait ): array {
			$traits = class_uses( $trait ) ?: [];

			foreach ( $traits as $trait ) {
				$traits += self::traitUsesRecursive( $trait );
			}

			return $traits;
		}

		public static function classUsesTrait( $object_or_class, string $class ): bool {
			return in_array( $class, self::classUsesRecursive( $object_or_class ) );
		}
	}

}